#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Collections;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

#region Global Enums

public enum anaKeltnerMAType {DEMA, EMA, HMA, HoltEMA, LinReg, SMA, SMMA, TEMA, TMA, VWMA, WMA, ZLEMA}
public enum anaKeltnerRangeType {SimpleRange, TrueRange}

#endregion

//This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
	/// Bollinger Bands are plotted at standard deviation levels above and below a moving average. Since standard deviation is a measure of volatility, the bands are self-adjusting: widening during volatile markets and contracting during calmer periods.
	/// </summary>
	[Description("Keltner Channels are plotted at a multiple of the simple or average true range above and below a moving average. Since range and average true range are a measure of volatility, the bands are self-adjusting: widening during volatile markets and contracting during calmer periods.")]
    public class anaKeltnerUniversal : Indicator
	{
		#region Variables
		private	double numMultiplier					= 2.0;
		private int period								= 20;
		private int bandPeriod							= 20;
		private int index								= 1;
		private bool smoothed							= false;
		private bool bullish							= true;
		private bool priorBull							= true;
		private Color upColor 							= Color.LimeGreen;
		private Color downColor 						= Color.Red;
		private int plotBWidth 							= 1;
		private PlotStyle plotBStyle 					= PlotStyle.Line;
		private DashStyle dlashBStyle 					= DashStyle.Solid;
		private int plotAWidth 							= 1;
		private PlotStyle plotAStyle 					= PlotStyle.Line;
		private DashStyle dlashAStyle 					= DashStyle.Solid;
		private int opacity								= 2;
		private anaKeltnerMAType channelMAType			= anaKeltnerMAType.SMA; 
		private anaKeltnerMAType midbandMAType			= anaKeltnerMAType.SMA; 
		private anaKeltnerRangeType selectedRangeType	= anaKeltnerRangeType.SimpleRange;
		private IDataSeries rangeSeries;
		private IDataSeries average;
		private IDataSeries smoothedAverage;
		private IDataSeries volatility;
		private IDataSeries smoothedVolatility;
		private Font textFont							= new Font("Arial", 12);
		private string errorData1						= "HoltEMA not found!";
		private string errorData2						= "SMMA not found!";
		#endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// </summary>
		
		protected override void Initialize()
		{
			Add(new Plot(Color.Gray, "Upper band"));
			Add(new Plot(Color.Gray, "Middle band"));
			Add(new Plot(Color.Gray, "Lower band"));
			Overlay		= true;
			PlotsConfigurable   = false;
		}

		protected override void OnStartUp()
		{
			switch (selectedRangeType)
			{
				case anaKeltnerRangeType.SimpleRange:
					rangeSeries = Range();
					break;
				case anaKeltnerRangeType.TrueRange:
					rangeSeries = ATR(Input,1);
					break;
			}
			switch (midbandMAType)
			{
				case anaKeltnerMAType.DEMA: 
					average = DEMA(Typical, Period);
					break;
				case anaKeltnerMAType.EMA: 
					average = EMA(Typical, Period);
					break;
				case anaKeltnerMAType.HMA: 
					average = HMA(Typical, Period);
					break;
				case anaKeltnerMAType.HoltEMA: 
					try
					{
						average = anaHoltEMA(Input, Period, 2*Period);
					}
					catch
					{			
						DrawTextFixed("errortag1", errorData1, TextPosition.Center, ChartControl.AxisColor, textFont, Color.Transparent,Color.Transparent,0);
					}
					break;
				case anaKeltnerMAType.LinReg: 
					average = LinReg(Typical, Period);
					break;
				case anaKeltnerMAType.SMA: 
					average = SMA(Typical, Period);
					break;
				case anaKeltnerMAType.SMMA: 
					try
					{
						average = anaSMMA(Typical, Period);
					}
					catch
					{			
						DrawTextFixed("errortag2", errorData2, TextPosition.Center, ChartControl.AxisColor, textFont, Color.Transparent,Color.Transparent,0);
					}
					break;
				case anaKeltnerMAType.TEMA: 
					average = TEMA(Typical, Period);
					break;
				case anaKeltnerMAType.TMA: 
					average = TMA(Typical, Period);
					break;
				case anaKeltnerMAType.VWMA: 
					average = VWMA(Typical, Period);
					break;
				case anaKeltnerMAType.WMA: 
					average = WMA(Typical, Period);
					break;
				case anaKeltnerMAType.ZLEMA: 
					average = ZLEMA(Typical, Period);
					break;
			}
			smoothedAverage = SMA(average,3);
			
			switch (channelMAType)
			{
				case anaKeltnerMAType.DEMA: 
					volatility = DEMA(rangeSeries,BandPeriod);
					break;
				case anaKeltnerMAType.EMA: 
					volatility = EMA(rangeSeries,BandPeriod);
					break;
				case anaKeltnerMAType.HMA: 
					volatility = HMA(rangeSeries,BandPeriod);
					break;
				case anaKeltnerMAType.HoltEMA: 
					try
					{
						volatility = anaHoltEMA(rangeSeries, BandPeriod, 2*BandPeriod);
					}
					catch
					{			
						DrawTextFixed("errortag3", errorData1, TextPosition.Center, ChartControl.AxisColor, textFont, Color.Transparent,Color.Transparent,0);
					}
					break;
				case anaKeltnerMAType.LinReg: 
					volatility = LinReg(rangeSeries,BandPeriod);
					break;
				case anaKeltnerMAType.SMA: 
					volatility = SMA(rangeSeries,BandPeriod);
					break;
				case anaKeltnerMAType.SMMA: 
					try
					{
						volatility = anaSMMA(rangeSeries,BandPeriod);
					}
					catch
					{			
						DrawTextFixed("errortag4", errorData2, TextPosition.Center, ChartControl.AxisColor, textFont, Color.Transparent,Color.Transparent,0);
					}
					break;
				case anaKeltnerMAType.TEMA: 
					volatility = TEMA(rangeSeries,BandPeriod);
					break;
				case anaKeltnerMAType.TMA: 
					volatility = TMA(rangeSeries,BandPeriod);
					break;
				case anaKeltnerMAType.VWMA: 
					volatility = VWMA(rangeSeries,BandPeriod);
					break;
				case anaKeltnerMAType.WMA: 
					volatility = WMA(rangeSeries,BandPeriod);
					break;
				case anaKeltnerMAType.ZLEMA: 
					volatility = ZLEMA(rangeSeries,BandPeriod);
					break;
			}
			smoothedVolatility = SMA(volatility,3);
			Plots[0].Pen.Width = plotBWidth;
			Plots[0].PlotStyle = plotBStyle;
			Plots[0].Pen.DashStyle = dlashBStyle;
			Plots[1].Pen.Width= plotAWidth;
			Plots[1].PlotStyle = plotAStyle;
			Plots[1].Pen.DashStyle = dlashAStyle;
			Plots[2].Pen.Width = plotBWidth;
			Plots[2].PlotStyle = plotBStyle;
			Plots[2].Pen.DashStyle = dlashBStyle;
			index = 1;
		}
		
		/// <summary>
		/// Called on each bar update event (incoming tick)
		/// </summary>
		protected override void OnBarUpdate()
		{
			double averageValue = 0;
			if (Smoothed)
				averageValue    = smoothedAverage[0];
			else
				averageValue    = average[0];
			double offset = 0;
			if (Smoothed)
				offset = NumMultiplier * smoothedVolatility[0];
			else
				offset = NumMultiplier * volatility[0];
            Upper.Set(averageValue + offset);
            Middle.Set(averageValue);
            Lower.Set(averageValue - offset);

			if (CurrentBar < Math.Max(Period, BandPeriod))
				return;
			if(FirstTickOfBar)
				priorBull = bullish;
			if (averageValue >=Middle[1])
			{
				PlotColors[0][0] = UpColor;
				PlotColors[1][0] = UpColor;
				PlotColors[2][0] = UpColor;
				if(!priorBull)
					index = CurrentBar;
				if (Opacity != 0)
					DrawRegion("keltner" + index, CurrentBar - index + 1, 0, Upper, Lower, Color.Transparent, UpColor, Opacity);
				bullish = true;
			}
				else
			{
				PlotColors[0][0] = DownColor;
				PlotColors[1][0] = DownColor;
				PlotColors[2][0] = DownColor;
				if(priorBull)
					index = CurrentBar;
				if (Opacity != 0)
					DrawRegion("keltner" + index, CurrentBar - index + 1, 0, Upper, Lower, Color.Transparent, DownColor, Opacity);
				bullish = false;
			}
		}

		#region Properties
		/// <summary>
		/// Get the upper value.
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Upper
		{
			get { return Values[0]; }
		}

		/// <summary>
		/// Get the middle value.
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Middle
		{
			get { return Values[1]; }
		}
		
		/// <summary>
		/// Gets the lower value.
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Lower
		{
			get { return Values[2]; }
		}

		/// <summary>
		/// </summary>
		[Description("Moving average Type for CenterLine")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayNameAttribute("MAType for CenterLine")]
		public anaKeltnerMAType MidbandMAType
		{
			get { return midbandMAType; }
			set { midbandMAType = value; }
		}

		/// <summary>
		/// </summary>
		[Description("Moving average Type for Outer Bands")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayNameAttribute("MAType for Volatility Bands")]
		public anaKeltnerMAType ChannelMAType
		{
			get { return channelMAType; }
			set { channelMAType = value; }
		}

		/// <summary>
		/// </summary>
		[Description("Simple or True Range")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayNameAttribute("Range Type")]
		public anaKeltnerRangeType SelectedRangeType
		{
			get { return selectedRangeType; }
			set { selectedRangeType = value; }
		}

		/// <summary>
		/// </summary>
		[Description("Number of standard deviations")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayNameAttribute("Multiplier for Std.Dev.")]
		public double NumMultiplier
		{
			get { return numMultiplier; }
			set { numMultiplier = Math.Max(0, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Number of Bars Used for Calculation of Center Line")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayNameAttribute("Period for CenterLine")]
		public int Period
		{
			get { return period; }
			set { period = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Number of Bars Used for Calculation of Volatility")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayNameAttribute("Period for Volatility Bands")]
		public int BandPeriod
		{
			get { return bandPeriod; }
			set { bandPeriod = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Smoothing for Center Line and Channels")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayNameAttribute("Smoothed Channels")]
		public bool Smoothed
		{
			get { return smoothed; }
			set { smoothed = value; }
		}

		/// <summary>
		/// </summary>
		[Description("Select color for Rising Average")]
		[Category("Plots")]
		[Gui.Design.DisplayName("Average Rising")]
		public Color UpColor
		{
			get { return upColor; }
			set { upColor = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string UpColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(upColor); }
			set { upColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("Select color for Falling Average")]
		[Category("Plots")]
		[Gui.Design.DisplayName("Average Falling")]
		public Color DownColor
		{
			get { return downColor; }
			set { downColor = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string DownColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(downColor); }
			set { downColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("Width for Bands.")]
		[Category("Plots")]
		[Gui.Design.DisplayNameAttribute("Line Width Bands")]
		public int PlotBWidth
		{
			get { return plotBWidth; }
			set { plotBWidth = Math.Max(1, value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("PlotStyle for Bands.")]
		[Category("Plots")]
		[Gui.Design.DisplayNameAttribute("Plot Style Bands")]
		public PlotStyle PlotBStyle
		{
			get { return plotBStyle; }
			set { plotBStyle = value; }
		}
		
		/// <summary>
		/// </summary>
		[Description("DashStyle for Bands.")]
		[Category("Plots")]
		[Gui.Design.DisplayNameAttribute("Dash Style Bands")]
		public DashStyle DlashBStyle
		{
			get { return dlashBStyle; }
			set { dlashBStyle = value; }
		} 
		/// <summary>
		/// </summary>
		[Description("Width for Average.")]
		[Category("Plots")]
		[Gui.Design.DisplayNameAttribute("Line Width Average")]
		public int PlotAWidth
		{
			get { return plotAWidth; }
			set { plotAWidth = Math.Max(1, value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("PlotStyle for Average.")]
		[Category("Plots")]
		[Gui.Design.DisplayNameAttribute("Plot Style Average")]
		public PlotStyle PlotAStyle
		{
			get { return plotAStyle; }
			set { plotAStyle = value; }
		}
		
		/// <summary>
		/// </summary>
		[Description("DashStyle for Average.")]
		[Category("Plots")]
		[Gui.Design.DisplayNameAttribute("Dash Style Average")]
		public DashStyle DlashAStyle
		{
			get { return dlashAStyle; }
			set { dlashAStyle = value; }
		} 

		/// <summary>
		/// </summary>
		[Description("Region Opacity")]
		[Category("Plots")]
		public int Opacity
		{
			get { return opacity; }
			set { opacity = Math.Max(0, value); }
		}
		#endregion
	}
}		

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private anaKeltnerUniversal[] cacheanaKeltnerUniversal = null;

        private static anaKeltnerUniversal checkanaKeltnerUniversal = new anaKeltnerUniversal();

        /// <summary>
        /// Keltner Channels are plotted at a multiple of the simple or average true range above and below a moving average. Since range and average true range are a measure of volatility, the bands are self-adjusting: widening during volatile markets and contracting during calmer periods.
        /// </summary>
        /// <returns></returns>
        public anaKeltnerUniversal anaKeltnerUniversal(int bandPeriod, anaKeltnerMAType channelMAType, anaKeltnerMAType midbandMAType, double numMultiplier, int period, anaKeltnerRangeType selectedRangeType, bool smoothed)
        {
            return anaKeltnerUniversal(Input, bandPeriod, channelMAType, midbandMAType, numMultiplier, period, selectedRangeType, smoothed);
        }

        /// <summary>
        /// Keltner Channels are plotted at a multiple of the simple or average true range above and below a moving average. Since range and average true range are a measure of volatility, the bands are self-adjusting: widening during volatile markets and contracting during calmer periods.
        /// </summary>
        /// <returns></returns>
        public anaKeltnerUniversal anaKeltnerUniversal(Data.IDataSeries input, int bandPeriod, anaKeltnerMAType channelMAType, anaKeltnerMAType midbandMAType, double numMultiplier, int period, anaKeltnerRangeType selectedRangeType, bool smoothed)
        {
            if (cacheanaKeltnerUniversal != null)
                for (int idx = 0; idx < cacheanaKeltnerUniversal.Length; idx++)
                    if (cacheanaKeltnerUniversal[idx].BandPeriod == bandPeriod && cacheanaKeltnerUniversal[idx].ChannelMAType == channelMAType && cacheanaKeltnerUniversal[idx].MidbandMAType == midbandMAType && Math.Abs(cacheanaKeltnerUniversal[idx].NumMultiplier - numMultiplier) <= double.Epsilon && cacheanaKeltnerUniversal[idx].Period == period && cacheanaKeltnerUniversal[idx].SelectedRangeType == selectedRangeType && cacheanaKeltnerUniversal[idx].Smoothed == smoothed && cacheanaKeltnerUniversal[idx].EqualsInput(input))
                        return cacheanaKeltnerUniversal[idx];

            lock (checkanaKeltnerUniversal)
            {
                checkanaKeltnerUniversal.BandPeriod = bandPeriod;
                bandPeriod = checkanaKeltnerUniversal.BandPeriod;
                checkanaKeltnerUniversal.ChannelMAType = channelMAType;
                channelMAType = checkanaKeltnerUniversal.ChannelMAType;
                checkanaKeltnerUniversal.MidbandMAType = midbandMAType;
                midbandMAType = checkanaKeltnerUniversal.MidbandMAType;
                checkanaKeltnerUniversal.NumMultiplier = numMultiplier;
                numMultiplier = checkanaKeltnerUniversal.NumMultiplier;
                checkanaKeltnerUniversal.Period = period;
                period = checkanaKeltnerUniversal.Period;
                checkanaKeltnerUniversal.SelectedRangeType = selectedRangeType;
                selectedRangeType = checkanaKeltnerUniversal.SelectedRangeType;
                checkanaKeltnerUniversal.Smoothed = smoothed;
                smoothed = checkanaKeltnerUniversal.Smoothed;

                if (cacheanaKeltnerUniversal != null)
                    for (int idx = 0; idx < cacheanaKeltnerUniversal.Length; idx++)
                        if (cacheanaKeltnerUniversal[idx].BandPeriod == bandPeriod && cacheanaKeltnerUniversal[idx].ChannelMAType == channelMAType && cacheanaKeltnerUniversal[idx].MidbandMAType == midbandMAType && Math.Abs(cacheanaKeltnerUniversal[idx].NumMultiplier - numMultiplier) <= double.Epsilon && cacheanaKeltnerUniversal[idx].Period == period && cacheanaKeltnerUniversal[idx].SelectedRangeType == selectedRangeType && cacheanaKeltnerUniversal[idx].Smoothed == smoothed && cacheanaKeltnerUniversal[idx].EqualsInput(input))
                            return cacheanaKeltnerUniversal[idx];

                anaKeltnerUniversal indicator = new anaKeltnerUniversal();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.BandPeriod = bandPeriod;
                indicator.ChannelMAType = channelMAType;
                indicator.MidbandMAType = midbandMAType;
                indicator.NumMultiplier = numMultiplier;
                indicator.Period = period;
                indicator.SelectedRangeType = selectedRangeType;
                indicator.Smoothed = smoothed;
                Indicators.Add(indicator);
                indicator.SetUp();

                anaKeltnerUniversal[] tmp = new anaKeltnerUniversal[cacheanaKeltnerUniversal == null ? 1 : cacheanaKeltnerUniversal.Length + 1];
                if (cacheanaKeltnerUniversal != null)
                    cacheanaKeltnerUniversal.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheanaKeltnerUniversal = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Keltner Channels are plotted at a multiple of the simple or average true range above and below a moving average. Since range and average true range are a measure of volatility, the bands are self-adjusting: widening during volatile markets and contracting during calmer periods.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.anaKeltnerUniversal anaKeltnerUniversal(int bandPeriod, anaKeltnerMAType channelMAType, anaKeltnerMAType midbandMAType, double numMultiplier, int period, anaKeltnerRangeType selectedRangeType, bool smoothed)
        {
            return _indicator.anaKeltnerUniversal(Input, bandPeriod, channelMAType, midbandMAType, numMultiplier, period, selectedRangeType, smoothed);
        }

        /// <summary>
        /// Keltner Channels are plotted at a multiple of the simple or average true range above and below a moving average. Since range and average true range are a measure of volatility, the bands are self-adjusting: widening during volatile markets and contracting during calmer periods.
        /// </summary>
        /// <returns></returns>
        public Indicator.anaKeltnerUniversal anaKeltnerUniversal(Data.IDataSeries input, int bandPeriod, anaKeltnerMAType channelMAType, anaKeltnerMAType midbandMAType, double numMultiplier, int period, anaKeltnerRangeType selectedRangeType, bool smoothed)
        {
            return _indicator.anaKeltnerUniversal(input, bandPeriod, channelMAType, midbandMAType, numMultiplier, period, selectedRangeType, smoothed);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Keltner Channels are plotted at a multiple of the simple or average true range above and below a moving average. Since range and average true range are a measure of volatility, the bands are self-adjusting: widening during volatile markets and contracting during calmer periods.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.anaKeltnerUniversal anaKeltnerUniversal(int bandPeriod, anaKeltnerMAType channelMAType, anaKeltnerMAType midbandMAType, double numMultiplier, int period, anaKeltnerRangeType selectedRangeType, bool smoothed)
        {
            return _indicator.anaKeltnerUniversal(Input, bandPeriod, channelMAType, midbandMAType, numMultiplier, period, selectedRangeType, smoothed);
        }

        /// <summary>
        /// Keltner Channels are plotted at a multiple of the simple or average true range above and below a moving average. Since range and average true range are a measure of volatility, the bands are self-adjusting: widening during volatile markets and contracting during calmer periods.
        /// </summary>
        /// <returns></returns>
        public Indicator.anaKeltnerUniversal anaKeltnerUniversal(Data.IDataSeries input, int bandPeriod, anaKeltnerMAType channelMAType, anaKeltnerMAType midbandMAType, double numMultiplier, int period, anaKeltnerRangeType selectedRangeType, bool smoothed)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.anaKeltnerUniversal(input, bandPeriod, channelMAType, midbandMAType, numMultiplier, period, selectedRangeType, smoothed);
        }
    }
}
#endregion
